/** LDAP Catalog User class for Grids
 * Description:		GridbusDirectory provides a user-friendly portal into the Java Naming and Directory Interface (JNDI).
 * It is designed for use with the Grids lab at the University of Melbourne and application to the HEP Belle/Atlas experiments.
 * Functions: Search for ldap entry via an attribute or with a filter string.
 * @author			Steve Melnikoff stevexm@cs.mu.oz.au
 * @version			1.0 October 2002
 */

//package cto_scheduler;

import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.event.*;
import javax.naming.ldap.*;
import javax.naming.spi.*;

public class GridbusDirectory {
	protected Hashtable env;
	protected DirContext ctx;
	protected String initial_context_factory;
	protected String provider_URL;
	protected String security_authentication;
	protected String security_principal;
	protected String security_credentials;
	protected String context;
	protected String filter;
	protected String returnedAttribute;
	protected int resultsLimit;
	protected int timeoutLimit;
	protected int searchScope;
	protected boolean secured;
	public NamingEnumeration searchResults = null;
	public static final int SUBTREESCOPE = 0;
	public static final int ONELEVELSCOPE = 1;
	public static final int OBJECTSCOPE = 2;
	public static final String INITIALCONTEXTFACTORY = "com.sun.jndi.ldap.LdapCtxFactory";
	public static final String PHYSICSLDAPURL = "";
	public static final int RESULTSLIMIT = 0;
	public static final int TIMEOUT = 0;
	public static final int MAX_NUM_ATTRIB = 25;
	
	/**
	* Constructor initializes the GridbusDirectory class using default parameters.
	* The default provider_URL, the ldap server, is the Univ. Melb. physics server
	* For searches, no limit on the number of results to return, and no timeout limit
	* Searches are initialized to follow the entire subtree, starting at the initial context/DN entry
	*/
	public GridbusDirectory() {
		initial_context_factory = INITIALCONTEXTFACTORY;
		provider_URL = PHYSICSLDAPURL;
		timeoutLimit = TIMEOUT;
		resultsLimit = RESULTSLIMIT;
		searchScope = SUBTREESCOPE;
		secured = false;
		env = new Hashtable();
	}
	public GridbusDirectory(String provider_URL) {
		initial_context_factory = INITIALCONTEXTFACTORY;
		this.provider_URL = provider_URL;
		timeoutLimit = TIMEOUT;
		resultsLimit = RESULTSLIMIT;
		searchScope = SUBTREESCOPE;
		secured = false;
		env = new Hashtable();
	}

	public DirContext getDirContext() {
		return ctx;
	}

	/**
	* initiate a connection to the service either using the current provider_URL
	* or connect using the supplied URL
	* @param provider_URL the initial ldap server to connect to
	* @exception NamingException
	*/
   public boolean connectServer() {
		try {
			ctx = new InitialDirContext(env);
		} catch (NamingException e) { 
			//System.out.println("FAILED...");
			return false;
		}
		return true;
	}
	
/*
	void connectServer(String provider_URL) throws NamingException {
		this.provider_URL = provider_URL;
		connectServer();
	}
*/
	
	/**
	* initialize the environment hashtable variable env before initiating a connection
	*/	
	void putEnvironment() {
			env.put(Context.INITIAL_CONTEXT_FACTORY,initial_context_factory);
			env.put(Context.PROVIDER_URL, provider_URL);	
	}
	/**
	* initialize the environment hashtable variable env for a secured connection
	*/	
	void putSecuredEnvironment() {
		env.put(Context.SECURITY_AUTHENTICATION,security_authentication);
		env.put(Context.SECURITY_PRINCIPAL,security_principal); // specify the username "cn=directorymanager" ?
		env.put(Context.SECURITY_CREDENTIALS,security_credentials);           // specify the password
	}
	
	/**
	* initiate a connection to the service either using the current provider_URL
	* or connect using the supplied URL. Can initiate a secured connection by setting the secured flag.
	* @param provider_URL the initial ldap server to connect to
	* @exception NamingException
	*/
	public boolean bind(String provider_URL) throws NamingException {
		this.provider_URL = provider_URL;
		return bind();
	}	
	public boolean bind()  { 
		putEnvironment();
		if(secured) putSecuredEnvironment();
		return connectServer(); 
	}
	
	/**
	* close the connection to the server, immediately, rather than waiting for garbage collection
	*/	
	public void disconnectServer() { 
		try {
			ctx.close();
		} catch (NamingException e) {
	    e.printStackTrace();	
    	}
    }    
	public void unbind() { disconnectServer(); }
		
	/**
	* Generic method to format the NamingEnumeration returned from a search.
	*/
	public  void formatResults(NamingEnumeration enum) throws Exception {
		int count=0;
		try {
	    	while (enum.hasMore()) {
				SearchResult sr = (SearchResult)enum.next();
				System.out.println("\n\n" + sr.getName());
				formatAttributes(sr.getAttributes());
				System.out.println("====================================================");
				count++;
			}
	   	System.out.println("Search returned "+ count+ " results");
		} catch (SizeLimitExceededException e) {
			System.out.println("Search results limit reached max "+ resultsLimit + " results");
			System.out.println("====================================================");
		} catch (NamingException e) {
		e.printStackTrace();	
    	} 
	}
	/**
	* Generic method to format the Attributes .Displays all the multiple values of
	* each Attribute in the Attributes
	*/
	 public  void formatAttributes(Attributes attrs) throws Exception{
		if (attrs == null) {
	    	System.out.println("This result has no attributes");
		} else {
	   		try {
				for (NamingEnumeration enum = attrs.getAll(); enum.hasMore();) {
		    		Attribute attrib = (Attribute)enum.next();
		    		System.out.print("\t" + attrib.getID());
		   		 for (NamingEnumeration e = attrib.getAll();e.hasMore();)
			 		System.out.println(" = " + e.next());
				}
	   		 } catch (NamingException e) {
			e.printStackTrace();
			}
		}
	}
		
	/**
	* given a search result extract and return from it the specified attribute
	* @param attrs, Attributes, the search result
	* @param attribStr, String, the attribute to extract
	*/
	public void attributeExtraction(Object o, String attribStr, Vector results) throws Exception {
		NamingEnumeration enum = (NamingEnumeration) o;
		try {
	    	while (enum.hasMore()) {
				SearchResult sr = (SearchResult)enum.next();
				extractAttribute(sr.getAttributes(), attribStr, results);
			}
		} catch (SizeLimitExceededException e) {
		} catch (NamingException e) {
			e.printStackTrace();	
    	} 
	}
	
	public void extractAttribute(Attributes attrs, String attribStr, Vector results) throws Exception {
		if (attrs == null) return;
	   	try {
			for (NamingEnumeration enum = attrs.getAll(); enum.hasMore();) {
		    	Attribute attrib = (Attribute)enum.next();
				if(attribStr.equals(attrib.getID()) ) 
		   		 	for (NamingEnumeration e = attrib.getAll();e.hasMore();) results.add(e.next());
			}
	   	} catch (NamingException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * create entry in the directory and store object, possibly null
	 * @param context, String, where on the tree to put object, full dn
	 * @param obj, Object, to store by means of supplied attributes, can be null
	 * @param attrib, Attributes, the attributes for this object
	 */
	public void store(String context, Object obj, Attributes attrib) {
		try {
			ctx.bind(context,obj,attrib);
		} catch (NamingException e) {
		}
	}
	/**
	* create entry in the directory and store object, possibly null
	* input via string arrays instead of Attribute object
	* @param context, String, where on the tree to put object, full dn
	* @param obj, Object to store by means of supplied attributes, can be null
	* @param objClass, String array of schema objects for defining attributes
	* @param attribID, String array of attribute names/IDs
	* @param attribVal, String array of attribute values corresponding to attribID array
	*/ 
	public void store(String context, Object obj, String[] objClass, String[] attribID, String[] attribVal) {
   		Attribute oc = new BasicAttribute("objectclass");
   		for(int i=0; i<objClass.length; i++) oc.add(objClass[i]);
   		Attributes attrib = new BasicAttributes(true);
		attrib.put(oc);
		for(int i=0; i<attribID.length; i++) attrib.put(attribID[i],attribVal[i]);
		store(context,obj,attrib);
	}
	
	public void addAttribute(Attribute attrib) {
		try {
			Attributes attrs = new BasicAttributes(true);
		 	attrs.put(attrib);
		 	ctx.modifyAttributes(context,DirContext.ADD_ATTRIBUTE,attrs);
		} catch (NamingException e) {
		}
	}
	
	public void addAttribute(String context, Attribute attrib) {
		this.context = context;
		addAttribute(attrib);
	}	
	public void addAttribute( String attribID, String attribVal) {
		try {
			Attributes attrs = new BasicAttributes(true);
			attrs.put(new BasicAttribute(attribID, attribVal) );
			ctx.modifyAttributes(context,DirContext.ADD_ATTRIBUTE,attrs);
		} catch (NamingException e) {
		}
	}
	public void addAttribute(String context, String attribID, String attribVal) {
		this.context = context;
		addAttribute(attribID,attribVal);
	}
	
	public void replaceAttribute(String attribID, String attribVal) {
		try {
			Attributes attrs = new BasicAttributes(true);
			attrs.put(new BasicAttribute(attribID, attribVal) );
			ctx.modifyAttributes(context,DirContext.REPLACE_ATTRIBUTE,attrs);
		} catch (NamingException e) {
		}
	}
	public void replaceAttribute(String context, String attribID, String attribVal) {
		this.context = context;
		replaceAttribute(attribID, attribVal);
	}
	
	public void deleteAttribute(String attribID) {
		try {
			Attributes attrs = new BasicAttributes(true);
			attrs.put(new BasicAttribute(attribID));
			ctx.modifyAttributes(context,DirContext.REMOVE_ATTRIBUTE,attrs);
		} catch (NamingException e) {
		}
	}
	public void deleteAttribute(String context, String attribID) {
		this.context = context;
		deleteAttribute(attribID);
	}
	
	/**
	 * Search for objects using filter or single attribute
	 * Create search controls and allow java objects to be returned in the results.
	 * @param context, String, where on the tree to begin searching
	 * @param attribID, String, which attribute to search for
	 * @param attribVal, String, value of attribute to search
	 * @return NamingEnumeration, results of search as a NamingEnumeration
	 */
	 public NamingEnumeration search(String context, String attribID, String attribVal) {
  		searchResults = null;
  		try {
		Attributes matchAttrs = new BasicAttributes(true);
		matchAttrs.put( new BasicAttribute(attribID, attribVal) );
 	 	searchResults =  ctx.search(context,matchAttrs);
		} catch (Exception e) { 
		}finally {
			return searchResults;
		}
	 }
 	public NamingEnumeration search(String context, String filter, String[] attribRtn, int limit) {
   		searchResults = null;
		try {
			SearchControls ctls = new SearchControls();
			resultsLimit = limit;
			if(resultsLimit>0) ctls.setCountLimit(resultsLimit);
			if(attribRtn != null) ctls.setReturningAttributes(attribRtn);
			ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
			searchResults =  ctx.search(context,filter,ctls);
		} catch (Exception e) {
		}finally {
			return searchResults;
		}
	 }
 
	 /**
	 * return an object using the supplied context, DN
	 * @param context, String, where on the tree to begin searching
	 * @return Object found at context or null
	 */
 	public Object lookup(String context) {
 		Object obj = null;
 		try {
 			obj = ctx.lookup(context);
 		} catch (Exception e) {
 		} finally {
 			return obj;
 		}
	 }
 
	public void renameDN(String DN, String newDN) throws NamingException { ctx.rename(DN, newDN); }
	
	 /**
	 * delete an object using the supplied context, DN
	 * @param context, String, where on the tree to delete entry
	 * @exception NamingException
	 */
	public void delete() throws NamingException { ctx.destroySubcontext(context); }
	public void delete(String context) throws NamingException { 
		this.context = context;
		delete();
	 }
	
	 /**
	 * set or get the URL for the LDAP server
	 * @param String provider_URL the URL for the server
	 * @return String, current URL for the LDAP server, provider_URL
	 */
	public String getProviderURL() { return provider_URL; }	 
	public void setProviderURL(String s) { provider_URL = s; }
	
	 /**
	 * set or get the initial_context_factory for the LDAP server, which is Sun?
	 * @param String initial_context_factory for the server
	 * @return String, current initial_context_factory for the LDAP server
	 */
	public String getInitialContextFactory() { return initial_context_factory; }
	public void setInitialContextFactory(String s) { initial_context_factory = s; }

	 /**
	 * set or get the context/DN for the LDAP entry
	 * @param String context, the LDAP entry DN
	 * @return String, current context for the LDAP entry DN
	 */
	public void setContext(String s) { context = s;  }
	public String getContext() { return context;  }
	
	 /**
	 * set or get the filter string for an attribute search in the form of a logical expression. RFC 2254 describes the syntax of search filters
	 * @param String filter, the search filter string
	 * @return String, filter, the current search filter string
 	*/
	public void setFilter(String s) { filter = s;  }	
	public String getFilter() { return filter;  }
	
	 /**
	 * set or get the control of which attribute to return after a filtered search
	 * @param String returnedAttribute the attribute to return
	 * @return String, the current attribute to return
	 */
	void setReturnedAttrib(String s) { returnedAttribute = s; }	
	public String getReturnedAttrib() { return returnedAttribute; }
	
	 /**
	 * set or get the control of how many entries to return after a filtered search
 	* @param int, resultsLimit, number of entries to return following a search
 	* @return int, resultsLimit current number of entries to return after a search
	 */
	public void setResultsLimit(int i) { resultsLimit = i; }	
	public int getResultsLimit() { return resultsLimit; }
																		
}


/* @(#)GridbusDirectory.java */
