import java.util.*;

/*
 * Location is the class where all the information about a particular grid is
 * stored, such as name, address, port, details and attributes. 
 *
 * @author Hussein Gibbins
 */
public class Location {

        public static final String LOC_COUNTRY = "loc_country";
        public static final String ONLINE = "online";
        public static final String MDS_OS_NAME = "Mds-Os-name";
        public static final String MDS_CPU_TOTAL_COUNT = "Mds-Cpu-Total-count";
        public static final String MDS_CPU_VENDOR = "Mds-Cpu-Vendor";
        public static final String MDS_COMPUTER_PLATFORM = "Mds-Computer-platform";
	public static final String EXTRA = "extra";

	public static final String DEFAULT_BASEDN = "Mds-Vo-name=local, o=grid";

	private boolean online;
	private String name;
	private String address;
	private int port;
	private String baseDN;
        private String cntryName;
        private String cntryCode;
	private String details;
	private Hashtable attributes = new Hashtable();

	private int x;
	private int y;

	/**
	 * Creates an instance of a location initialising it with the given attributes.
	 */
	public Location(String name, String address, int port, String details, int x, int y) {

		this.name = name;
		this.address = address;
		this.port = port;
		this.details = details;
		this.x = x;
		this.y = y;
		setAttribute(EXTRA, "");
		this.baseDN = DEFAULT_BASEDN;
	}

	/**
	 * Sets the status of this location to either being online or offline.
	 *
	 * @param online whether the location is online or not.
	 */
	public void setOnline(boolean online) {
		this.online = online;
	}

	/**
	 * Checks whether this location is online.
	 *
	 * @return true if the location is online, false otherwise.
	 */
	public boolean isOnline() {
		return online;
	}

	/**
	 * Sets the name of this location.
	 *
	 * @param this location name.
	 */
        public void setName(String name) {
                this.name = name;
        }

	/**
	 * Gets the name of this location.
	 *
	 * @return this location name.
	 */
        public String getName() {
                return name;
        }

	/**
	 * Sets the address which will be used to query the resource to which this location corresponds.
	 *
	 * @param the address of the resource for this location.
	 */
        public void setAddress(String add) {
                this.address = add;
        }

	/**
	 * Gets the address used to query the resource to which this location corresponds.
	 *
	 * @return the address of the resource for this location.
	 */
        public String getAddress() {
                return address;
        }

	/**
	 * Sets the country data for this location.  More specifically, it takes a string of form
	 * "countryCode:countryName" and sets the country code and country name details for this
	 * location.
	 *
	 * @param cntry the country data information.
	 * @see getCountryCode
	 * @see getCountryName
	 */
        public void setCountryData(String cntry) {

                StringTokenizer st = new StringTokenizer(cntry, ":");
                if (st.hasMoreTokens()) {
                        cntryCode = st.nextToken();
                        cntryName = st.nextToken();
                }
        }

	/**
	 * Gets the name of the country associated with this location.
	 *
	 * @return the country name.
	 */
        public String getCountryName() {
                return cntryName;
        }

	/**
	 * Gets the country code information corresponding to the country associated with this location.
	 *
	 * @return the country code corresponding to the country for this location.
	 */
        public String getCountryCode() {
                return cntryCode;
        }

	/**
	 * Sets the port used to query this location.
	 *
	 * @param the port number.
	 */
        public void setPort(int port) {
                this.port = port;
        }

	/**
	 * Gets the port used to query this location.
	 *
	 * @return the port number.
	 */
        public int getPort() {
                return port;
        }

        /**
         * Sets the base dn used when making MDS queries to this location.
	 *
         * @param the base dn.
         */
	public void setBaseDN(String baseDN) {
		this.baseDN = baseDN;
	}

        /**
         * Gets the base dn used when making MDS queries to this location.
         *
	 * @return the base dn.
         */
	public String getBaseDN() {
		return baseDN;
	}

        /**
         * Sets the details text for this location.
	 *
         * @param details about this location.
         */
        public void setDetails(String details) {
                this.details = details;
        }

        /**
         * Gets the detail text for this location.
         *
	 * @return the detail text.
         */
        public String getDetails() {
                return details;
        }

        /**
         * Gets the X coordinate used to position the location on the testbed map.
         *
	 * @return X coordinate.
         */
	public int getX() {
		return x;
	}

        /**
         * Gets the Y coordinate used to position the location on the testbed map.
         *
         * @return Y coordinate.
         */
	public int getY() {
		return y;
	}

        /**
         * Sets the X coordinate used to position the location on the testbed map.
         *
         * @param X coordinate.
         */
	public void setX(int x) {
		this.x = x;
	}

        /**
         * Sets the Y coordinate used to position the location on the testbed map.
         *
         * @param Y coordinate.
         */
	public void setY(int y) {
		this.y = y;
	}

	/**
	 * Sets the value for a given attribute for this location.
	 *
	 * @param attribute the name of the attribute.
	 * @param value the value to be set for the given attribute.
	 */
        public void setAttribute(String attribute, String value) {
                attributes.put(attribute, value);
        }

	/**
	 * Gets the value for a given attribute.
	 *
	 * @return the value for the given attribute.
	 */
        public String getAttribute(String attribute) {
                return (String) attributes.get(attribute);
        }

        /**
         * Checks whether this location contains any attribute with the given value.
         *
         * @return whether the value is present.
         */
        public boolean checkValue(String value) {
                return attributes.contains(value);
        }

        /**
         * Gets a list of all the attribute names contained in this location.
         *
         * @return a list of attribute names.
         */
	public Enumeration getAttributeNames() {
		return attributes.keys();
	}

        /**
         * Gets a list of all the attribute values contained in this location.
         *
         * @return a list of attribute values.
         */
	public Enumeration getAttributeValues() {
		return attributes.elements();
	}

        /**
         * Provids a string representation of the attribute names and values contained in this location.
         *
         * @return a string representation of attribute names and values.
         */
	public String getAttributeData() {

                String data = "";
                String[] codes = TestbedManager.getAttributeCodes();

                data += "Attribute:Value \n";
                for (int i=0; i < codes.length; i++) {
                        data += codes[i] + ":" + getAttribute(codes[i]) + "\n";
                }
                return data;
	}
}

