import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;


/**
 * This class does the managing of the testbed components.  It performs query and search operations, provides a load and
 * save mechanism for grid resources/locations.
 *
 * @author Hussein Gibbins.
 */
public class TestbedManager {

	private static LocationList testbeds = new LocationList();
	private static final String DEFAULT_SEP = "|||";
        private Properties props = new Properties();
// need to get web app to know where to get its resources from
        private static final String propsFilename = "testbed.properties";
        private static String dataSavePathFile = "GridList.txt";
//        private String dataSavePath = "";
        private String appType = "normal";
	private static String testbedName = "default";
	private static String webappPath = "";

	private static boolean initialised = false;
	private static int managers = 0;
	private static int updates = 0;

	private static String[] attribute_codes = {
		"Mds-Os-name",
		"Mds-Cpu-Total-count",
		"Mds-Cpu-Model",
		"Mds-Cpu-Vendor",
		"Mds-Computer-platform",
		"Mds-Fs-Total-sizeMB"
		};

        private static String[] attribute_names = {
                "Operating System",
                "Total CPUs",
		"CPU Model",
                "CPU Vendor",
                "Computer Platform",
		"Fs Total Size (MB)"
                };

	String ldapPrefix = "ldap://";
	String defaultPort = "2135";

	String context = "Mds-Vo-name=local, o=grid";
	String filter = "(Mds-host-hn=*)";

	LocationList testbeds_search = null;

	/**
	 * Creates an instance of the TestbedManager.  Initialises data structures, loads locations.
	 */
	public TestbedManager() {
		managers++;
		initialise();
	}

	/**
	 * Re-initialises data structures.
	 */
	public void updateLocationList() {
		//testbeds = new LocationList();
		//testbeds.clear();
		while (testbeds.size() > 0) {
			testbeds.remove(testbeds.getNewLocation());
		}
		initialise();
		updates++;
		readLocations();
		testbeds.complete();
	}

	/**
	 * Returns the number of instances of this class that have been created.
	 *
	 * @return number of instances.
	 */
	public int howManyManagers() {
		return managers;
	}

	/**
	 * Returns the number of times this testbed manager has been updated.
	 *
	 * @return number of updates.
	 */
	public int howManyUpdates() {
		return updates;
	}

        private void initialise() {
                try {

//                        props.load(new FileInputStream(this.getClass().getResource(propsFilename).getPath()));
props.load(new FileInputStream(System.getProperty("user.dir")+"/"+propsFilename));

                        ldapPrefix = props.getProperty("ldap_prefix");
                        defaultPort = props.getProperty("default_port");
                        testbedName = props.getProperty("testbed_name");
                        webappPath = props.getProperty("webapp_path");
                        if (webappPath.equals("")) {
                                String fullPath = this.getClass().getResource("../../../" + dataSavePathFile).getPath();
                                webappPath = fullPath.substring(0,fullPath.length() - dataSavePathFile.length());
                        }
                } catch (Exception e) {
                        System.out.println("FAILED TO OPEN PROPERTIES FILE: " + propsFilename);
                }

                if (!initialised) {
                        updates++;
                        readLocations();
                }
                initialised = true;
        }

	private void readFromDB() {
		// queryAllDirect() removed from here.. affects web-app.
	}

	/**
	 * Queries the location on the top of the location list (usually meaning the location currently
	 * being interacted with).  The information retrieved is stored in the location for later use.
	 */
	public void queryTop() {
		queryDirect((Location) testbeds.getNewLocation());	
	}

	/**
	 * Queries all locations in the location list.  Equivalent to performing queryTop on each location.
	 */
	public void queryAllDirect() {

		Location loc;

                for (int i = 0; i < testbeds.size(); i++) {
                        loc = (Location) testbeds.elementAt(i);
                        queryDirect(loc);
                }
	}

	private void queryDirect(Location loc) {

                GridbusDirectory gbd = new GridbusDirectory();
                Attributes attrRes;
                Attribute attr;
                SearchResult sr;
                NamingEnumeration nameEnum;
		
		try {
                        //System.out.println("Connecting... (" + loc.getName() + ")");
                        boolean online = gbd.bind(ldapPrefix + loc.getAddress() + ":" + loc.getPort());
                        loc.setOnline(online);
                        nameEnum = gbd.search(loc.getBaseDN(), filter, attribute_codes, 0);

                        if (nameEnum != null && online) {

                                while (nameEnum.hasMore()) {

                                        sr = (SearchResult)nameEnum.next();
                                        attrRes = sr.getAttributes();
                                        for (int j = 0; j < attribute_codes.length; j++) {
                                                attr = extractAttribute(attrRes, attribute_codes[j]);
                                                if (attr != null) {
                                                        loc.setAttribute(attribute_codes[j], (String)attr.get());
                                                } else {
                                                        loc.setAttribute(attribute_codes[j], "NULL");
						}
                                        }
                                }
                        } else {
				for (int j = 0; j < attribute_codes.length; j++) {
					loc.setAttribute(attribute_codes[j], "NULL");
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		testbeds.refresh();
	}

	private Attribute extractAttribute(Attributes attrs, String attrName) {
		return attrs.get(attrName);
	}

	/**
	 * Searches the location list for the given attribute and value pair.  Sets the search list
	 * within this class.
	 *
	 * @param attribute the attribute to search for.
	 * @param value the value of the attribute.
	 */
	public void search(String attribute, String value) {

		boolean contains = false;
		String tmpValue;
		Location loc;

		testbeds_search = new LocationList();

		if (attribute.equalsIgnoreCase("") || attribute.equalsIgnoreCase("*")) {
			doAnyValueSearch(value);
			return;
		}

		for (int i = 0; i < testbeds.size(); i++) {
			loc = (Location) testbeds.elementAt(i);
			tmpValue = loc.getAttribute(attribute);
			//here we could use tmpValue.matches(...) in order to make use
			//of regular expressions in search. (Java 1.4).
			if (tmpValue != null && tmpValue.equalsIgnoreCase(value)) {
				doSearchFound(loc, tmpValue);
			}
		}
	}

        /**
         * Searches for a specific host address (location address).  Sets the search list.
         *
         * @param address the host address.
         */
	public void searchHost(String address) {

		testbeds_search = new LocationList();
		Location loc;

		for (int i = 0; i < testbeds.size(); i++) {
			loc = (Location) testbeds.elementAt(i);
			if ((loc.getAddress()).equalsIgnoreCase(address)) {
				doSearchFound(loc, address);
				break;
			}
		}
	}

        /**
         * Searches locations for a specific country.  Sets the search list.
         *
         * @param countryCode the country code for the desired country.
         */
	public void searchCountry(String countryCode) {

		testbeds_search = new LocationList();
		Location loc;

		for (int i=0; i < testbeds.size(); i++) {
			loc = (Location) testbeds.elementAt(i);
			if ((loc.getCountryCode()).equalsIgnoreCase(countryCode)) {
				doSearchFound(loc, countryCode);
			}
		}
	}

	private void doAnyValueSearch(String value) {

		boolean contains = false;
		Location loc;

		for (int i = 0; i < testbeds.size(); i++) {
			loc = (Location) testbeds.elementAt(i);
			contains = loc.checkValue(value);
			if (contains) {
				doSearchFound(loc, value);
			}
		}
	}

	private void doSearchFound(Location loc, String value) {
		testbeds_search.add(loc);
		loc.setAttribute(Location.EXTRA, value);

	}

        /**
         * Gets the name of the testbed.
         *
         * @return the name of the testbed.
         */
	public static String getTestbedName() {
		return testbedName;
	}

        /**
         * Gets the path to the web application used to load elements of the testbed.
         *
         * @return the path to the web application.
         */
	public static String getWebappPath() {
		return webappPath;
	}

        /**
         * Gets the list of locations associated with this testbed manager.
         *
         * @return the location list.
         */
	public LocationList getLocationList() {
		return testbeds;
	}


        /**
         * Gets the list of locations which satisfied the most recent search.
	 *
	 * note: concurrency issue with implementation.
         *
         * @return list of locations satisfying most recent search.
         */
	public LocationList getSearchTestbeds() {
		if (testbeds_search == null) {
			testbeds_search = testbeds;
		}
		return testbeds_search;
	}

        /**
         * Resets the list of locations satisfying most current search.
         */
	public void resetSearch() {
		testbeds_search = testbeds;
		Location loc;
		for (int i = 0; i < testbeds.size(); i++) {
			loc = (Location) testbeds.elementAt(i);
			loc.setAttribute(Location.EXTRA, "");
		}
	}


	// should modify to use XML
	/*
	 * reads locations from file.
	 */
	private void readLocations() {

                //File file = new File(dataSavePath);
                File file = new File(webappPath+dataSavePathFile);
                if (file.exists())      {
                        try{
                                BufferedReader br = new BufferedReader(new FileReader(file));
                                String s;
                                while ( (s = br.readLine()) != null )   {
                                        s = s.trim();
                                        StringTokenizer st = new StringTokenizer(s,DEFAULT_SEP);

                                        String name = st.nextToken();
                                        String address = st.nextToken();
                                        int port = Integer.parseInt(st.nextToken());
                                        String cntryData = st.nextToken();
					String currentBaseDN = st.nextToken();
                                        String details = st.nextToken();

                                        //just read more of the line.
                                        int xLoc = Integer.parseInt(st.nextToken());
                                        int yLoc = Integer.parseInt(st.nextToken());

                                        Location theNewLoc = new Location(name, address, port, details, xLoc, yLoc);
                                        theNewLoc.setCountryData(cntryData);
					theNewLoc.setBaseDN(currentBaseDN);
                                        testbeds.add(theNewLoc);
                                }
                        }
                        catch (Exception e){
                                System.out.println("Error during parsing of the Resource Specification file");
                                return;
                        }
                } else {
                        System.out.println("file does not exist");
                        return;
                }
	}


        /**
         * Writes the details of your locations to file.
         */
        public void saveLocations() {

                File tmpFile = new File("temp.txt");
                try{
                        tmpFile.createNewFile();
                }
                catch (IOException io){
                        io.getMessage();
                }

                try{
                        FileWriter writer = new FileWriter(tmpFile);
                        for (int i=0;i<testbeds.size();i++){
                                Location l = (Location) testbeds.elementAt(i);
                                writer.write(l.getName() + DEFAULT_SEP);
                                writer.write(l.getAddress()  + DEFAULT_SEP);
                                writer.write(l.getPort()  + DEFAULT_SEP);
                                writer.write(l.getCountryCode() + ":" + l.getCountryName() + DEFAULT_SEP);
                                writer.write(l.getBaseDN() + DEFAULT_SEP);
                                writer.write(l.getDetails() + DEFAULT_SEP);
                                writer.write(l.getX() + DEFAULT_SEP + l.getY());
                                writer.write("\n");
                        }
                        writer.close();

				File file = new File(webappPath+dataSavePathFile);
				File bakFile = new File(file.getParent() + "/GridList.bak");
				if (bakFile.exists()) {
					bakFile.delete();
				}
				file.renameTo(bakFile);
				if (file.exists()) {
					file.delete();
				}
                        boolean success = tmpFile.renameTo(file);
				System.out.println("rename:"+success);

                } catch (IOException io) {
				System.out.println("ERROR ERROR ERROR");
                }
        }

	/**
	 *
	 *
	 *
	 */
	public static String[] getAttributeCodes() {
		return attribute_codes;
	}

        /**
         *
         *
         *
         */
	public static String[] getAttributeNames() {
		return attribute_names;
	}

        /**
         *
         *
         *
         */
	public Iterator getValueSet(String attribute) {

		LinkedHashSet set = new LinkedHashSet();
		Location loc;

		for (int i = 0; i < testbeds.size(); i++) {
			loc = testbeds.elementAt(i);
			set.add(loc.getAttribute(attribute));
		}

		return set.iterator();
	}
}

